/** 
 * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
 * 
 */

package hwb1;

import javacard.framework.*;
import javacardx.annotations.*;
import static hwb1.MyAppletStrings.*;

/**
 * Applet class
 * 
 * @author <user>
 */
@StringPool(value = {
        @StringDef(name = "Package", value = "hwb1"),
        @StringDef(name = "AppletName", value = "MyApplet"),
        @StringDef(name = "AuthorName", value = "Manuel") },
        // Insert your strings here
        name = "MyAppletStrings")
public class MyApplet extends Applet {
    private byte[] storage = null;

    /**
     * Installs this applet.
     * 
     * @param bArray  the array containing installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the parameter data in bArray
     */
    public static void install(byte[] bArray, short bOffset, byte bLength) {
        new MyApplet();
    }

    /**
     * Only this class's install method should create the applet object.
     */
    protected MyApplet() {
        register();
    }

    /**
     * Processes an incoming APDU.
     * 
     * @see APDU
     * @param apdu the incoming APDU
     */
    @Override
    public void process(APDU apdu) {
        short length;
        byte[] buffer = apdu.getBuffer();
        
        if (buffer[ISO7816.OFFSET_CLA] != 0x80) {
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
        } else {
            byte instruction = buffer[ISO7816.OFFSET_INS];

            switch (instruction) {
                case 0x00:
                case 0x04:
                    byte[] response;
                    length = apdu.setOutgoing();

                    if (instruction == 0x00) {
                        response = AuthorName;
                        length = (short)response.length;
                    } else {
                        response = storage;

                        if (length > response.length) {
                            length = (short) response.length;
                        }
                    }

                    apdu.setOutgoingLength(length);
                    apdu.sendBytesLong(response, (short) 0, length);
                    break;
                case 0x02:
                    length = apdu.setIncomingAndReceive();

                    if (length > 20) {
                        ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
                    } else {
                        Util.arrayCopy(buffer, ISO7816.OFFSET_CDATA, storage, (short) 0, length);
                    }
                    break;
                default:
                    ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
                    break;
            }
        }
    }
}
